/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal.emfstore;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.emf.emfstore.server.LocationProvider;

/**
 * Base implementation of a {@link LocationProvider} which uses the Eclipse
 * workspace location as base location.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @levd.rating RED Rev:
 */
public abstract class EclipseWorkspaceLocationProviderBase implements
		LocationProvider {

	/** Store the workspace location. */
	private IProject workspaceProject;

	/** Stores the backup location. */
	private IProject backupProject;

	/** Stores initilization flag. */
	private boolean initialized = false;

	/** Initialize the locations. */
	private void initialize() {
		IWorkspaceRoot workspaceRoot = ResourcesPlugin.getWorkspace().getRoot();
		this.workspaceProject = workspaceRoot
				.getProject(getWorkspaceProjectName());
		if (!workspaceProject.exists()) {
			try {
				workspaceProject.create(null);
			} catch (CoreException e) {
				printCreationError(getWorkspaceProjectName(), e);
			}
		}
		this.backupProject = workspaceRoot.getProject(getBackupProjectName());
		if (!backupProject.exists()) {
			try {
				backupProject.create(null);
			} catch (CoreException e) {
				printCreationError(getBackupProjectName(), e);
			}
		}
		initialized = true;
	}

	/** Prints error message to standard error. */
	private final void printCreationError(String projectName, Exception ex) {
		System.err.println("Could not create project " + projectName);
		ex.printStackTrace(System.err);
	}

	/** Returns the name of the workspace project. */
	protected abstract String getWorkspaceProjectName();

	/** Returns the name of the backup project. */
	protected abstract String getBackupProjectName();

	/** {@inheritDoc} */
	@Override
	public final String getWorkspaceDirectory() {
		if (!initialized) {
			initialize();
		}
		return workspaceProject.getLocation().toString();
	}

	/** {@inheritDoc} */
	@Override
	public final String getBackupDirectory() {
		if (!initialized) {
			initialize();
		}
		return backupProject.getLocation().toString();
	}
}
