/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.services;

import org.conqat.lib.commons.collections.UnmodifiableList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.fortiss.tooling.kernel.interfaces.IPrototypeProvider;
import org.fortiss.tooling.kernel.internal.PrototypeService;

/**
 * The prototype service provides registration and access to model element
 * prototypes.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @levd.rating RED Rev:
 */
public interface IPrototypeService {

	/** Returns the singleton instance of the service. */
	public static final IPrototypeService INSTANCE = new PrototypeService();

	/** Returns the list of all registered prototype providers. */
	UnmodifiableList<IPrototypeProvider> getPrototypeProviders();

	/** Returns the list of all prototypes registered any provider. */
	UnmodifiableList<Prototype> getAllPrototypes();

	/**
	 * Prototypes consist of a generic description and preconfigured
	 * {@link EObject}.
	 */
	public static final class Prototype {
		/** The generic prototype name. */
		private final String name;
		/** the preconfigure {@link EObject}. */
		private final EObject prototype;

		/** Constructor. */
		public Prototype(String name, EObject prototype) {
			this.name = name;
			this.prototype = prototype;
		}

		/** Returns the name. */
		public String getName() {
			return name;
		}

		/** Returns the prototype instance (NOT a copy!). */
		public EObject getPrototype() {
			return prototype;
		}

		/**
		 * Returns a copy of the prototype. This method is potentially
		 * expensive.
		 */
		public EObject getPrototypeCopy() {
			return EcoreUtil.copy(prototype);
		}
	}
}
