/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.util;

import org.conqat.ide.commons.ui.logging.LoggingUtils;
import org.conqat.lib.commons.reflect.ReflectionUtils;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecp.common.model.ECPWorkspaceManager;
import org.eclipse.emf.ecp.common.model.workSpaceModel.ECPProject;
import org.eclipse.emf.emfstore.client.model.ProjectSpace;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.model.IProjectRootElement;

/**
 * Utility methods for accessing {@link IProjectRootElement}s.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @levd.rating RED Rev:
 */
public final class ProjectRootElementUtils {

	/**
	 * Returns the model element's ECP project or <code>null</code> .
	 */
	public static ECPProject getECPProject(EObject element) {
		try {
			return ECPWorkspaceManager.getInstance().getWorkSpace()
					.getProject(element);
		} catch (Exception e) {
			LoggingUtils.error(ToolingKernelActivator.getDefault(),
					"Unable to find ECP project!", e);
		}
		return null;
	}

	/**
	 * Returns the instance of a project root element corresponding to the given
	 * class.
	 */
	public static <T extends IProjectRootElement> T getRootElement(
			EObject element, Class<T> clazz) {
		ECPProject project = getECPProject(element);
		return ReflectionUtils.pickInstanceOf(clazz, ((ProjectSpace) project
				.getRootObject()).getProject().getAllModelElements());
	}
}
